package command.engine.md25;

import java.util.ArrayList;

import command.engine.EngineCommand;
import command.engine.EngineCommandFactory;
import command.engine.Move;
import command.engine.Off;
import command.engine.On;
import command.engine.SpecificCommand;
import command.engine.VirtualEngineCommand;
import exception.ArgumentMismatchException;
import exception.UnknownCommandTypeException;

public class MD25CommandFactory implements EngineCommandFactory {

	@Override
	public Move createMove(ArrayList<Object> args) throws ArgumentMismatchException {
		try {
			return new MD25Move((Byte)args.get(0),(Byte)args.get(1));
		} catch(RuntimeException e) {
			throw new ArgumentMismatchException("Move",e);
		}
	}

	@Override
	public On createOn(ArrayList<Object> args) throws ArgumentMismatchException {
		return new MD25On();
	}

	@Override
	public Off createOff(ArrayList<Object> args) throws ArgumentMismatchException {
		return new MD25Off();
	}

	@Override
	public SpecificCommand createSpecificCommand(ArrayList<Object> args) throws ArgumentMismatchException {
		try{
			return new MD25SpecificCommand((byte[])args.get(0));
		} catch(RuntimeException e) {
			throw new ArgumentMismatchException("Specific",e);
		}
	}

	@Override
	public EngineCommand createCommand(VirtualEngineCommand cmd,
			ArrayList<Object> args) throws ArgumentMismatchException, UnknownCommandTypeException {
		switch(cmd){
		case ON:
			return createOn(args);
		case OFF:
			return createOff(args);
		case MOVE:
			return createMove(args);
		case SPECIFIC:
			return createSpecificCommand(args);
		default: throw new UnknownCommandTypeException(cmd.toString());
		}
	}

}
