package command.engine.md25;

import module.engine.Engine;
import module.engine.md25.MD25EngineInstruction;
import module.engine.md25.MD25EngineTranslator;

import command.engine.Move;
import exception.CommandFailureException;
import exception.UnsupportedCommandException;

public class MD25Move implements Move {
	
	private byte speed;
	private byte direction;

	public MD25Move(byte speed, byte direction) {
		super();
		this.speed = speed;
		this.direction = direction;
	}

	public byte getSpeed() {
		return speed;
	}

	public void setSpeed(byte speed) {
		this.speed = speed;
	}

	public byte getDirection() {
		return direction;
	}

	public void setDirection(byte direction) {
		this.direction = direction;
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 5086643689432454427L;

	@Override
	public void execute(Engine engine) throws CommandFailureException,UnsupportedCommandException {
		
		int speed1 = 128 + speed;
		int speed2 = 128 + speed;

		if(direction > 0)
			speed1 += direction;
		else if (direction < 0)
			speed2 -= direction;
		
		if(speed1 > 255)
			speed1 = 255;
		else if(speed1 < 0)
			speed1 = 0;
				
		if(speed2 > 255)
			speed2 = 255;
		else if(speed2 < 0)
			speed2 = 0;
				
		byte[] speed1_bytes = { (byte) speed1 };
		byte[] speed2_bytes = { (byte) speed2 };
		
		try {
			engine.send(MD25EngineTranslator.makeCmd(MD25EngineInstruction.SETSPEED1, speed1_bytes));
			engine.send(MD25EngineTranslator.makeCmd(MD25EngineInstruction.SETSPEED2, speed2_bytes));
		} catch (Exception e) {
			throw new CommandFailureException(this.getClass().getName(),e);
		}
	}

}