package command.engine.md25;

import java.util.ArrayList;

import command.engine.Battery;
import command.engine.Direction;
import command.engine.EngineQuery;
import command.engine.EngineQueryFactory;
import command.engine.SpecificQuery;
import command.engine.Speed;
import command.engine.VirtualEngineQuery;
import exception.ArgumentMismatchException;
import exception.UnknownCommandTypeException;

public class MD25QueryFactory implements EngineQueryFactory {

	@Override
	public Battery createBattery(ArrayList<Object> args) throws ArgumentMismatchException {
		return new MD25Battery();
	}

	@Override
	public Speed createSpeed(ArrayList<Object> args) throws ArgumentMismatchException {
		return new MD25Speed();
	}

	@Override
	public Direction createDirection(ArrayList<Object> args) throws ArgumentMismatchException {
		return new MD25Direction();
	}

	@Override
	public SpecificQuery createSpecificQuery(ArrayList<Object> args) throws ArgumentMismatchException {
		try{
			return new MD25SpecificQuery((byte[])args.get(0));
		} catch(RuntimeException e) {
			throw new ArgumentMismatchException("Specific",e);
		}
	}

	@Override
	public EngineQuery createQuery(VirtualEngineQuery q, ArrayList<Object> args) throws ArgumentMismatchException, UnknownCommandTypeException {
		switch(q){
		case BATTERY:
			return createBattery(args);
		case DIRECTION:
			return createDirection(args);
		case SPEED:
			return createSpeed(args);
		case SPECIFIC:
			return createSpecificQuery(args);
		default: throw new UnknownCommandTypeException(q.toString());
		}
	}

}
