package controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dto.OutputDTO;
import exception.FakeException;

import module.IndependentModule;
import util.FrequencyOptions;
import util.VisibilityOptions;

public abstract class Controller implements Runnable {
	protected Map<IndependentModule<?,?>,Thread> threads;

	@Override
	public void run() {
		deployModules(createModules());
		try {
			while(true) {
				control();
				try { 
					Thread.sleep(FrequencyOptions.ControlPeriod);
				} catch(InterruptedException e) {}
			}
		} catch(Exception e) {
			if(!(e instanceof FakeException))
				e.printStackTrace();
			stopModules();
		}
	}
	
	
	protected abstract List<IndependentModule<?, ?>> createModules();

	/**
	 * Control function that reads the modules outputs, decides the following actions and writes them in their corresponding modules inputs. 
	 * @throws Exception
	 */
	protected abstract void control() throws Exception;


	protected void deployModules(List<IndependentModule<?, ?>> modules) {
		threads = new HashMap<IndependentModule<?,?>,Thread>();
		for(IndependentModule<?,?> module : modules) {
			Thread t = new Thread(module);
			threads.put(module,t);
			t.start();
		}
	}
	
	protected void checkStatus(OutputDTO output, boolean critical)
			throws Exception {
		if (output.dead) {
			if (critical) {
				if (output.exception != null) {
					throw output.exception;
				} else {
					if (VisibilityOptions.verbose || VisibilityOptions.debug)
						System.out.println("Critical module has terminated successfully. Program will close.");
					throw new FakeException();
				}
			} else if (output.exception != null) {
				if (VisibilityOptions.verbose || VisibilityOptions.debug)
					System.err.println("Non-critical module has crashed due to an "
									+ output.exception.getClass().getName()
									+ ".");
			} else {
				if (VisibilityOptions.verbose || VisibilityOptions.debug)
					System.out.println("Module has terminated successfully.");
			}

		}
	}
	

	protected abstract void stopModules();
	
}
