package controller;

import java.util.ArrayList;
import java.util.List;

import util.CommunicationOptions;
import util.VisibilityOptions;

import module.IndependentModule;
import module.communication.udp.UDPClient;
import module.engine.md25.MD25Engine;
import controller.interpreter.CommEngInterpreter;
import dto.Synchronized;
import dto.communication.ClientOutputDTO;
import dto.engine.EngineInputDTO;
import dto.engine.EngineOutputDTO;

public class MD25SlaveController extends SlaveController {
	protected Synchronized<EngineInputDTO> Eoutput = new Synchronized<EngineInputDTO>(new EngineInputDTO());
	protected Synchronized<EngineOutputDTO> Einput = new Synchronized<EngineOutputDTO>(new EngineOutputDTO());
	protected MD25Engine engine = new MD25Engine(Eoutput,Einput);
	protected CommEngInterpreter interpreter = new CommEngInterpreter(client,engine);
	
	public MD25SlaveController() {
		client = new UDPClient(Coutput,Cinput,CommunicationOptions.clientPort);
	}

	@Override
	protected List<IndependentModule<?, ?>> createModules() {
		List<IndependentModule<?, ?>> ret = new ArrayList<IndependentModule<?, ?>>();
		ret.addAll(super.createModules());
		ret.add(engine);
		return ret;
	}

	@Override
	protected void stopModules() {
		super.stopModules();
		EngineInputDTO Ei = new EngineInputDTO();
		Ei.kill=true;
		Eoutput.write(Ei);
	}

	@Override
	protected void control() throws Exception {
		ClientOutputDTO clientReport = Cinput.read();
		EngineOutputDTO engineReport = Einput.read();
		
		if(clientReport.updated) {
			checkStatus(clientReport, true);
			EngineInputDTO in = interpreter.interpret(clientReport);
			if(in != null)
				Eoutput.write(in);
			else if(VisibilityOptions.verbose) System.out.println("Packet could not be deciphered.");
		} else {
			EngineInputDTO in = interpreter.followUp();
			if(in != null)
				Eoutput.write(in);
		}
		if(engineReport.updated)
			checkStatus(engineReport, true);
	}

}
