package controller;

import java.util.ArrayList;
import java.util.List;
import module.IndependentModule;
import module.communication.udp.UDPServer;
import module.engine.md25.MD25Engine;
import util.CommunicationOptions;
import util.TypeUtil;
import util.VisibilityOptions;
import dto.Synchronized;
import dto.communication.ServerInputDTO;
import dto.communication.ServerOutputDTO;
import dto.engine.EngineInputDTO;
import dto.engine.EngineOutputDTO;

public abstract class SimpleMD25MasterController extends MasterController {
	protected Synchronized<EngineInputDTO> Eoutput = new Synchronized<EngineInputDTO>(new EngineInputDTO());
	protected Synchronized<EngineOutputDTO> Einput = new Synchronized<EngineOutputDTO>(new EngineOutputDTO());
	protected MD25Engine engine = new MD25Engine(Eoutput,Einput);
	
	public SimpleMD25MasterController(){
		server = new UDPServer(Soutput,Sinput,CommunicationOptions.serverPort);
	}

	@Override
	protected List<IndependentModule<?, ?>> createModules() {
		List<IndependentModule<?, ?>> ret = new ArrayList<IndependentModule<?, ?>>();
		if(CommunicationOptions.slave != null)
			ret.addAll(super.createModules());
		if(!CommunicationOptions.remoteOnly)
			ret.add(engine);
		return ret;
	}

	@Override
	protected void stopModules() {
		if(CommunicationOptions.slave != null)
			super.stopModules();
		if(!CommunicationOptions.remoteOnly){
			EngineInputDTO Ei = new EngineInputDTO();
			Ei.kill=true;
			Eoutput.write(Ei);
		}
	}

	@Override
	protected void control() throws Exception {
		ServerOutputDTO serverReport = Sinput.read();
		EngineOutputDTO engineReport = Einput.read();
		
		if(serverReport.updated)
			checkStatus(serverReport, CommunicationOptions.remoteOnly);
		if(engineReport.updated)
			checkStatus(engineReport, CommunicationOptions.slave==null);
	}
	
	protected void instructEngineAndServer(EngineInputDTO in) {
		if (in != null) {
			if (!CommunicationOptions.remoteOnly)
				Eoutput.write(in);
			if (CommunicationOptions.slave != null) {
				ServerInputDTO Si = new ServerInputDTO();
				Si.message = TypeUtil.object2byte(in);
				if(Si.message != null) {
					Si.receiver = CommunicationOptions.slave;
					Soutput.write(Si);
				}
				else if(VisibilityOptions.verbose) System.out.println("Packet could not be transmitted.");
			}
		}
	}
}
