package controller;

import java.util.ArrayList;
import java.util.List;
import module.IndependentModule;
import module.communication.udp.UDPServer;
import module.vision.webcam.WebCameraVision;
import util.CommunicationOptions;
import controller.interpreter.WCamMD25Interpreter;
import dto.Synchronized;
import dto.engine.EngineInputDTO;
import dto.vision.VisionInputDTO;
import dto.vision.VisionOutputDTO;

public class WCamMD25MasterController extends SimpleMD25MasterController {
	protected Synchronized<VisionInputDTO> Voutput = new Synchronized<VisionInputDTO>(new VisionInputDTO());
	protected Synchronized<VisionOutputDTO> Vinput = new Synchronized<VisionOutputDTO>(new VisionOutputDTO());
	protected WebCameraVision camera = new WebCameraVision(Voutput,Vinput);
	protected WCamMD25Interpreter interpreter = new WCamMD25Interpreter(camera,engine);
	
	public WCamMD25MasterController(){
		server = new UDPServer(Soutput,Sinput,CommunicationOptions.serverPort);
	}

	@Override
	protected List<IndependentModule<?, ?>> createModules() {
		List<IndependentModule<?, ?>> ret = new ArrayList<IndependentModule<?, ?>>();
		ret.addAll(super.createModules());
		ret.add(camera);
		return ret;
	}

	@Override
	protected void stopModules() {
		super.stopModules();
		VisionInputDTO Vi = new VisionInputDTO();
		Vi.kill=true;
		Voutput.write(Vi);
	}

	@Override
	protected void control() throws Exception {
		super.control();
		VisionOutputDTO cameraReport = Vinput.read();
		if(cameraReport.updated) {
			checkStatus(cameraReport, true);
			EngineInputDTO in = interpreter.interpret(cameraReport);
			if(in != null)
				instructEngineAndServer(in);
		} else {
			EngineInputDTO in = interpreter.followUp();
			if(in != null)
				instructEngineAndServer(in);
		}
	}
}
