package controller;

import java.util.ArrayList;
import java.util.List;
import controller.interpreter.WKeyMD25Interpreter;
import controller.interpreter.WKeyWKeyInterpreter;
import util.CommunicationOptions;
import dto.Synchronized;
import dto.engine.EngineInputDTO;
import dto.keyboard.KeyboardInputDTO;
import dto.keyboard.KeyboardOutputDTO;
import module.IndependentModule;
import module.communication.udp.UDPServer;
import module.keyboard.windowed.WindowedKeyboard;

public class WKeyMD25MasterController extends SimpleMD25MasterController {
	protected Synchronized<KeyboardInputDTO> Koutput = new Synchronized<KeyboardInputDTO>(new KeyboardInputDTO());
	protected Synchronized<KeyboardOutputDTO> Kinput = new Synchronized<KeyboardOutputDTO>(new KeyboardOutputDTO());
	protected WindowedKeyboard keyboard = new WindowedKeyboard(Koutput,Kinput);
	protected WKeyMD25Interpreter Einterpreter = new WKeyMD25Interpreter(keyboard,engine);
	protected WKeyWKeyInterpreter Kinterpreter = new WKeyWKeyInterpreter(keyboard,keyboard);
	
	public WKeyMD25MasterController(){
		server = new UDPServer(Soutput,Sinput,CommunicationOptions.serverPort);
	}

	@Override
	protected List<IndependentModule<?, ?>> createModules() {
		List<IndependentModule<?, ?>> ret = new ArrayList<IndependentModule<?, ?>>();
		ret.addAll(super.createModules());
		ret.add(keyboard);
		return ret;
	}

	@Override
	protected void stopModules() {
		super.stopModules();
		KeyboardInputDTO Ki = new KeyboardInputDTO();
		Ki.kill=true;
		Koutput.write(Ki);
	}

	@Override
	protected void control() throws Exception {
		super.control();
		KeyboardOutputDTO keyboardReport = Kinput.read();
		if(keyboardReport.updated) {
			checkStatus(keyboardReport, true);
			EngineInputDTO in = Einterpreter.interpret(keyboardReport);
			if(in != null)
				instructEngineAndServer(in);
			KeyboardInputDTO kinstr = Kinterpreter.interpret(keyboardReport);
			if(kinstr != null)
				Koutput.write(kinstr);
		} else {
			EngineInputDTO in = Einterpreter.followUp();
			if(in != null)
				instructEngineAndServer(in);
		}
	}
}
