package controller.interpreter;

import java.util.ArrayList;
import java.util.Map;

import util.VisionOptions;

import com.googlecode.javacv.cpp.opencv_core.CvPoint;
import command.engine.VirtualEngineCommand;
import module.engine.md25.MD25Engine;
import module.vision.webcam.WebCameraVision;
import dto.engine.EngineInputDTO;
import dto.vision.VisionOutputDTO;

public class WCamMD25Interpreter extends Interpreter<EngineInputDTO,VisionOutputDTO,WebCameraVision,MD25Engine> {
	
	private static final short SpeedLimit 					= 70;
	private static final short SpeedIncrement				= 2;
	private static final short CentroidDeviationLimit		= 20;
	private static final short SpeedCorrectionFactor		= 20;
	private static final short DeviationCorrectionFactor	= 4;
	private static final short CurveSpeedLimit				= 25;
	private int speed;
	private int deviation;
	private EngineInputDTO lastInstr;
	

	public WCamMD25Interpreter(WebCameraVision interpretedModule,MD25Engine instructedModule) {
		super(interpretedModule, instructedModule);
	}

	@Override
	public EngineInputDTO interpret(VisionOutputDTO input) throws Exception {
		int imageCenter = VisionOptions.camWidth/2;
		
		@SuppressWarnings("unchecked")
		Map<CvPoint,Integer> centroids = (Map<CvPoint, Integer>) input.centroids.centroids;
		
		if (centroids != null && !centroids.isEmpty()) {

			CvPoint centroid = null;
			int area = 0;
			for (CvPoint p : centroids.keySet()) {
				int pArea = centroids.get(p);
				if (pArea > area) {
					centroid = p;
					area = pArea;
				} else if (pArea == area) {
					int diff = imageCenter - centroid.x();
					int pdiff = imageCenter - p.x();
					if (pdiff < diff)
						centroid = p;
				}
			}

			int x_diff = centroid.x() - imageCenter;

			if (Math.abs(x_diff) > CentroidDeviationLimit) {
				if (speed > CurveSpeedLimit)
					speed = speed
							- Math.round((Math.abs(x_diff) / SpeedCorrectionFactor));
				else
					speed = CurveSpeedLimit;
				deviation = x_diff / DeviationCorrectionFactor;
			} else {
				if (speed > SpeedLimit)
					speed = SpeedLimit;
				else
					speed += SpeedIncrement;
				deviation = 0;
			}
		}
		EngineInputDTO dto = new EngineInputDTO();
		ArrayList<Object> args = new ArrayList<Object>();
		args.add(new Byte((byte) speed));
		args.add(new Byte((byte) deviation));
		dto.command = instructedModule.MakeCommand( VirtualEngineCommand.MOVE, args);
		
		lastInstr = dto;
		
		return dto;
	}
	
	@Override
	public EngineInputDTO followUp() throws Exception {
		return lastInstr;
	}

}
