package controller.interpreter;

import java.awt.event.KeyEvent;
import java.util.ArrayList;

import command.engine.VirtualEngineCommand;
import module.engine.md25.MD25Engine;
import module.keyboard.windowed.WindowedKeyboard;
import dto.engine.EngineInputDTO;
import dto.keyboard.KeyboardOutputDTO;
import exception.ArgumentMismatchException;
import exception.UnknownCommandTypeException;
import exception.UnrelatedKeyException;

public class WKeyMD25Interpreter extends Interpreter<EngineInputDTO,KeyboardOutputDTO,WindowedKeyboard,MD25Engine>{
	
	private static final short SpeedLimit 		= 100;
	private static final short SpeedIncrement	= 6;
	private static final short DeviationStep	= 25;
	private int speed;
	private int deviation;
	
	public WKeyMD25Interpreter(WindowedKeyboard interpretedModule, MD25Engine instructedModule) {
		super(interpretedModule, instructedModule);
	}

	@Override
	public EngineInputDTO interpret(KeyboardOutputDTO input) throws UnrelatedKeyException, ArgumentMismatchException, UnknownCommandTypeException {
			int key = (Integer) input.key.key;
			
			EngineInputDTO dto = new EngineInputDTO();
			if(key == KeyEvent.VK_UP)
				speed = (speed + SpeedIncrement) <= SpeedLimit ? speed + SpeedIncrement : SpeedLimit;
			else if(key == KeyEvent.VK_DOWN)
				speed = (speed - SpeedIncrement) >= -SpeedLimit ? speed - SpeedIncrement : -SpeedLimit;
			else if(key == KeyEvent.VK_LEFT)
					deviation=-DeviationStep;
			else if(key == KeyEvent.VK_RIGHT)
					deviation=DeviationStep;
			else if(key == KeyEvent.VK_ESCAPE){}
			else throw new UnrelatedKeyException(""+key);
			
			return encodeMovement(dto);
			
	}

	private EngineInputDTO encodeMovement(EngineInputDTO instr) throws ArgumentMismatchException, UnknownCommandTypeException {
		ArrayList<Object> args = new ArrayList<Object>();
		args.add(new Byte((byte) speed));
		args.add(new Byte((byte) deviation));
		instr.command = instructedModule.MakeCommand(VirtualEngineCommand.MOVE, args);
		return instr;
	}
	
	@Override
	public EngineInputDTO followUp() throws Exception {
		deviation = 0;
		return encodeMovement(new EngineInputDTO());
	}

}
