package module;

import java.util.ArrayList;

import dto.InputDTO;
import dto.OutputDTO;
import dto.Synchronized;
import util.VisibilityOptions;
import exception.CriticalException;

public abstract class IndependentModule<I extends InputDTO,O extends OutputDTO> implements Runnable {
	
	protected Synchronized<I> input;
	protected Synchronized<O> output;
	protected I instructions;
	protected O report;
	protected boolean on;
	
	public boolean isOn() {
		return on;
	}

	protected IndependentModule(Synchronized<I> input,Synchronized<O> output) {
		this.input = input;
		this.output = output;
	}
	
	protected abstract void start() throws Exception;
	protected abstract void stop() throws Exception;
	protected abstract void act() throws Exception;
	protected abstract void perceive() throws Exception;
	public abstract long getPeriod();
	
	@Override
	public void run() {
		Exception CoD = null;
		try {
			start();
			while(true) {
				instructions = input.read();
				if(instructions.kill) break;
				if(on)
					if(instructions.updated)
						act();
					else perceive();
				try { Thread.sleep(getPeriod()); }catch(Exception e){}
			}
			stop();
		} catch(Exception e){
			CoD = e;
			boolean critical=false;
			ArrayList<Exception> flatExceptions = new ArrayList<Exception>();
			while(e != null) {
				if(e instanceof CriticalException) critical=true;
				flatExceptions.add(e);
				e =  (e.getCause() instanceof Exception) ? (Exception) e.getCause() : null;
			}
			if(VisibilityOptions.debug || critical) CoD.printStackTrace();
			else if(VisibilityOptions.verbose)
				for(int i = flatExceptions.size()-1; i >= 0 ; i--) {				
					System.err.println(flatExceptions.get(i).getMessage());
				}
			if(critical) try { stop(); } catch (Exception e2) {}
		}
		report = output.read();
		report.dead=true;
		report.exception=CoD;
		output.write(report);
	}
}
			
