package module.communication.udp;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;

import module.communication.Client;

import util.CommunicationOptions;
import util.SocketUtil;

import dto.Synchronized;
import dto.communication.ClientInputDTO;
import dto.communication.ClientOutputDTO;
import exception.ClientFailureException;

public class UDPClient extends Client {
	private DatagramSocket udpS;

	public UDPClient(Synchronized<ClientInputDTO> input,Synchronized<ClientOutputDTO> output,int port) {
		super(input, output,port);
	}

	@Override
	protected void perceive() throws ClientFailureException {
		try {
			DatagramPacket d = SocketUtil.receive(udpS,CommunicationOptions.maxPacketSize);
			report = new ClientOutputDTO();
			report.message = d.getData();
			report.sender = d.getPort() + "@" + d.getAddress().getHostName();
			output.write(report);
		} catch (IOException e) {
			throw new ClientFailureException("port - "+port,e);
		}
	}
	
	@Override
	protected void start() throws Exception{
		try {
			udpS = new DatagramSocket(port);
		} catch (SocketException e) {
			throw new ClientFailureException("port - "+port,e);
		}
		on=true;
	}
	
	@Override
	protected void stop() throws Exception {
		udpS.close();
		on=false;
	}
}
