package module.engine;

import java.util.ArrayList;

import module.IndependentModule;

import util.FrequencyOptions;

import command.Command;
import command.VirtualCommand;
import command.engine.EngineCommand;
import command.engine.EngineCommandFactory;
import command.engine.EngineQuery;
import command.engine.EngineQueryFactory;
import command.engine.VirtualEngineCommand;
import command.engine.VirtualEngineQuery;
import dto.Synchronized;
import dto.engine.EngineInputDTO;
import dto.engine.EngineOutputDTO;

import exception.ArgumentMismatchException;
import exception.CommandFailureException;
import exception.CommunicationException;
import exception.EngineNotFoundException;
import exception.InUseException;
import exception.QueryFailureException;
import exception.SetupException;
import exception.UnknownCommandTypeException;
import exception.UnsupportedCommandException;
import exception.UnsupportedQueryException;

public abstract class Engine extends IndependentModule<EngineInputDTO,EngineOutputDTO> {
	
	protected EngineCommandFactory cfactory;
	protected EngineQueryFactory qfactory;
	
	protected Engine(Synchronized<EngineInputDTO> input,Synchronized<EngineOutputDTO> output) {
		super(input, output);
	}
	
	@Override
	public long getPeriod() {
		return FrequencyOptions.EnginePeriod;
	}
	
	@Override
    public abstract void stop();
	
	@Override
    public abstract void start() throws InUseException, SetupException, EngineNotFoundException;
	
	@Override
    protected void perceive() {}
    
    public abstract void send(byte[] b) throws CommunicationException;
    public abstract byte[] receive(int bytes) throws CommunicationException;
    
    public Command MakeCommand(VirtualCommand vc, ArrayList<Object> args) throws ArgumentMismatchException, UnknownCommandTypeException {
    	if(vc instanceof VirtualEngineQuery)
			return qfactory.createQuery((VirtualEngineQuery)vc, args);
		else if(vc instanceof VirtualEngineCommand)
			return cfactory.createCommand((VirtualEngineCommand)vc, args);
		else throw new UnknownCommandTypeException(vc.getClass().getName());
    }
    
    @Override
    protected void act() throws UnknownCommandTypeException, CommandFailureException, UnsupportedCommandException, QueryFailureException, UnsupportedQueryException, ArgumentMismatchException {
    	Command cmd = instructions.command;
    	if(cmd instanceof EngineQuery) {
        	double result = ((EngineQuery)cmd).execute(this);
        	report = new EngineOutputDTO();
        	report.answer = result;
        	output.write(report);
       	} else if(cmd instanceof EngineCommand) {
       			((EngineCommand)cmd).execute(this);
       	} else throw new UnknownCommandTypeException(cmd.getClass().getName());
    }
}

