package module.engine.md25;

import java.io.IOException;

import module.engine.Engine;

import command.engine.md25.MD25CommandFactory;
import command.engine.md25.MD25QueryFactory;
import dto.Synchronized;
import dto.engine.EngineInputDTO;
import dto.engine.EngineOutputDTO;

import util.VisibilityOptions;
import util.SerialUtil;
import exception.CommunicationException;
import exception.EngineNotFoundException;
import exception.InUseException;
import exception.PortNotFoundException;
import exception.SetupException;
import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;

public class MD25Engine extends Engine {
	
    private CommPortIdentifier	portId;
    private SerialPort	      	serialPort;
	private String portname;
    
    public MD25Engine(Synchronized<EngineInputDTO> input,Synchronized<EngineOutputDTO> output) {
    	super(input,output);
    	qfactory = new MD25QueryFactory();
		cfactory = new MD25CommandFactory();
	}
	
	public MD25Engine(Synchronized<EngineInputDTO> input,Synchronized<EngineOutputDTO> output,String portname) {
		super(input,output);
		qfactory = new MD25QueryFactory();
		cfactory = new MD25CommandFactory();
		this.portname = portname; 
	}
	
	@Override
	public void stop() {
		serialPort.close();
		if(VisibilityOptions.verbose) System.out.println("Engine port closed.");
		on=false;
	}

	@Override
	public void start() throws InUseException, SetupException, EngineNotFoundException {
		try {
			portId = portname == null ? SerialUtil.getSerialPort() : SerialUtil.getSerialPort(portname);
		} catch (PortNotFoundException e) {
			throw portname == null ? new EngineNotFoundException(e) : new EngineNotFoundException(portname,e);
		}
		if(VisibilityOptions.verbose) System.out.println("Engine port found: " + portId.getName() + ".");
		try {
			serialPort = (SerialPort) portId.open("MD25Engines", 2000); //why 2000?? copied that from example. The owner is the MD25Engines class.
			if(VisibilityOptions.verbose) System.out.println("Engine port opened.");
			serialPort.setSerialPortParams(38400, 
			       SerialPort.DATABITS_8,
			       SerialPort.STOPBITS_2, 
			       SerialPort.PARITY_NONE);
		}catch(PortInUseException e){
			throw new InUseException(portId.getName(),e);
		} catch (UnsupportedCommOperationException e) {
			throw new SetupException(portId.getName(),e);
		}
		if(VisibilityOptions.verbose) System.out.println("Engine port setup complete.");
		if(VisibilityOptions.verbose) System.out.println("Engine port connection established.");
    	on = true;
	}

	public SerialPort getSerialPort() {
		return serialPort;
	}

	@Override
	public void send(byte[] b) throws CommunicationException {
		try {
			SerialUtil.send(serialPort, b);
		} catch (IOException e) {
			throw new CommunicationException(this.getClass().getName(),e);
		}
	}

	@Override
	public byte[] receive(int bytes) throws CommunicationException {
		try {
			return SerialUtil.receive(getSerialPort(),bytes);
		} catch (IOException e) {
			throw new CommunicationException(this.getClass().getName(),e);
		}
	}
	
}
