package module.keyboard.windowed;

import java.awt.event.KeyEvent;

import module.keyboard.Key;
import module.keyboard.Keyboard;

import util.KeyboardOptions;

import com.googlecode.javacv.CanvasFrame;
import com.googlecode.javacv.cpp.opencv_core.IplImage;

import static com.googlecode.javacv.cpp.opencv_highgui.*;

import dto.Synchronized;
import dto.keyboard.KeyboardInputDTO;
import dto.keyboard.KeyboardOutputDTO;

public class WindowedKeyboard extends Keyboard {
	private CanvasFrame controlWindow;
	private IplImage image;
	
	public WindowedKeyboard(Synchronized<KeyboardInputDTO> input, Synchronized<KeyboardOutputDTO> output) {
		super(input, output);
	}
	
	@Override
	protected void start() throws Exception {
		controlWindow = new CanvasFrame("Control Window");
		image = cvLoadImage(KeyboardOptions.ImageFile);
		if(image != null)
			controlWindow.showImage(image);
		on=true;
	}

	@Override
	protected void stop() throws Exception {
			controlWindow.dispose();
			on=false;
	}

	@Override
	protected void perceive() throws Exception {
		if(image != null)
			controlWindow.showImage(image);
		KeyEvent keyEv = controlWindow.waitKey(KeyboardOptions.WaitTime);
			
		int key = 0;
		
		if(keyEv != null) {
				key = keyEv.getKeyCode();
		}
		
		if(key == KeyEvent.VK_UP || 
				key == KeyEvent.VK_DOWN || 
				key == KeyEvent.VK_LEFT || 
				key == KeyEvent.VK_RIGHT || 
				key == KeyEvent.VK_ESCAPE) {
			report = new KeyboardOutputDTO();
			report.key = new Key<Integer>(key);
			output.write(report);
		}
			
		}
}
