package module.vision;

import org.joda.time.DateTime;

import util.FrequencyOptions;
import util.VisibilityOptions;

import dto.Synchronized;
import dto.vision.VisionInputDTO;
import dto.vision.VisionOutputDTO;
import exception.VisionFailureException;
import module.IndependentModule;

public abstract class Vision extends IndependentModule<VisionInputDTO,VisionOutputDTO>{
	private long t0;
	private long now;
	private int fps;
	
	protected Vision(Synchronized<VisionInputDTO> input,Synchronized<VisionOutputDTO> output) {
		super(input, output);
		t0 = new DateTime().getMillis();
	}
	
	/**
	 * Process an image having in count some parameters specified
	 * in an input DTO.
	 * 
	 */
	protected abstract Image<?> perceiveVision() throws VisionFailureException ;
	
	protected abstract CentroidMap<?> processVision(Image<?> image);

	@Override
	protected void perceive() throws VisionFailureException {
		report = new VisionOutputDTO();
		report.image = perceiveVision();
		report.centroids = processVision(report.image);
		if(!report.centroids.centroids.isEmpty())
			output.write(report);
		
		if(VisibilityOptions.print_fps) {
			long time_elapsed = (new DateTime().getMillis() - t0)/1000;				
			if(time_elapsed == now)
				fps++;
			else {
				System.out.println(fps);
				now = time_elapsed;
				fps=0;
			}
		}
		
	}

	@Override
	protected void act() {}

	@Override
	public long getPeriod() {
		return FrequencyOptions.VisionPeriod;
	}

}
