package util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;

import exception.PortNotFoundException;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

public class SerialUtil {
	
	public static CommPortIdentifier getSerialPort() throws PortNotFoundException{
		ArrayList<CommPortIdentifier> ports = findPorts(CommPortIdentifier.PORT_SERIAL);
		if(ports.isEmpty())
			throw new PortNotFoundException("Serial port");
		return ports.get(0);
	}
	
	public static CommPortIdentifier getSerialPort(String name) throws PortNotFoundException{
		ArrayList<CommPortIdentifier> ports = findPorts(CommPortIdentifier.PORT_SERIAL);
		if(ports.isEmpty())
			throw new PortNotFoundException("Serial port");
		for(CommPortIdentifier p : ports)
			if(p.getName().compareTo(name) == 0) return p;
		throw new PortNotFoundException("Serial port " + name);
	}
	
	
	@SuppressWarnings("unchecked")
	public static ArrayList<CommPortIdentifier> findPorts(int portType) {
		ArrayList<CommPortIdentifier> ports = new ArrayList<CommPortIdentifier>();
		Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();
		while ( portEnum.hasMoreElements() ) 
		{
			CommPortIdentifier portIdentifier = portEnum.nextElement();
			if(portIdentifier.getPortType() == portType)
				ports.add(portIdentifier);
		}
		return ports;
	}
	
    public static void send(SerialPort p, byte[] cmd) throws IOException {
    	p.getOutputStream().write(cmd);
    }
	
	public static byte[] receive(SerialPort p, int bytes) throws IOException {
		byte[] ret = new byte[bytes];
		int read = p.getInputStream().read(ret, 0, bytes);
		for(int i = read; i < bytes; i++)
			ret[i] = 0;
		return ret;
	}

}
