package util;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;

public class SocketUtil {

	public static void SequentialBroadcast(DatagramSocket udpS, byte[] message, ArrayList<String> arrayList) throws UnknownHostException, IOException {
		for(String s : arrayList)
			simpleSend(udpS,message,s);
	}
	
	/**
	 * Sends a message to the a host's process listening on the 
	 * specified port, through an UDP socket.
	 * 
	 * @param udpS - the UDP socket through which bytes are sent.
	 * @param message - the message to send.
	 * @param receiver - the String identifying the port + host name.
	 * @throws UnknownHostException - if no IP address for the host could be found.
	 * @throws IOException - if an I/O error occurs.
	 */
	public static void simpleSend(DatagramSocket udpS, byte[] message, String receiver) throws IOException {
		
		String[] receiverInfo = receiver.split("@");
		
		String	host;
		int		port;
		// should we verify receiverInfo length?
		
		host = receiverInfo[1];
		port = Integer.parseInt(receiverInfo[0]);
		
		InetSocketAddress host_addr = new InetSocketAddress(host, port);
		DatagramPacket packet = new DatagramPacket(message,message.length,host_addr);
		
		udpS.send(packet);

	}
	
	public static void send(DatagramSocket udpS, byte[] message, String receiver) throws IOException {
		if(!receiver.equals(CommunicationOptions.broadcastKeyword))
			SocketUtil.simpleSend(udpS,message,receiver);
		else
			SocketUtil.SequentialBroadcast(udpS,message,CommunicationUtil.getCommNames());
	}

	/**
	 * Receives an UDP packet from an UDP socket.
	 * 
	 * @param udpS - the UDP socket from which packet is received.
	 * @return the UDP packet containing the receiving data.
	 * @throws IOException - if an I/O error occurs.
	 */
	public static DatagramPacket receive(DatagramSocket udpS, int max_packet_size) throws IOException {

		byte[] data = new byte[max_packet_size];
		DatagramPacket packet = new DatagramPacket(data,data.length);
		udpS.receive(packet);
		return packet;
	}

}
