package util;

import static com.googlecode.javacv.cpp.opencv_imgproc.CV_CHAIN_APPROX_SIMPLE;
import static com.googlecode.javacv.cpp.opencv_imgproc.CV_RETR_LIST;
import static com.googlecode.javacv.cpp.opencv_imgproc.cvFindContours;
import static com.googlecode.javacv.cpp.opencv_imgproc.cvMinAreaRect2;

import java.util.HashMap;
import java.util.Map;
import com.googlecode.javacpp.Loader;
import com.googlecode.javacv.OpenCVFrameGrabber;
import com.googlecode.javacv.cpp.opencv_core.CvBox2D;
import com.googlecode.javacv.cpp.opencv_core.CvContour;
import com.googlecode.javacv.cpp.opencv_core.CvMemStorage;
import com.googlecode.javacv.cpp.opencv_core.CvPoint;
import com.googlecode.javacv.cpp.opencv_core.CvPoint2D32f;
import com.googlecode.javacv.cpp.opencv_core.CvSeq;
import com.googlecode.javacv.cpp.opencv_core.IplImage;

import exception.CameraCaptureException;


public class WebCameraUtil {
	
	/**
	 * Tries to capture camera device /dev/video{CAM_ID}
	 * 
	 * @return	a camera device handler, if it succeeds opening it.
	 * @throws CameraCaptureException
	 */
	public static OpenCVFrameGrabber captureCamera(int cam,int width,int height) throws CameraCaptureException {
		
		OpenCVFrameGrabber web_camera = new OpenCVFrameGrabber(cam);
	    
		web_camera.setImageWidth(width);
		web_camera.setImageHeight(height);
		
	    try {
			web_camera.start();
		} catch (Exception e) {
			throw new CameraCaptureException(""+cam,e);
		}
	    
		return web_camera;
	}

	/**
	 * Computes central points of minimum area rectangles which
	 * includes the respective contours of the polygons found in the frame.
	 * 
	 * @param bwImg
	 * @param MAX_RECT_AREA
	 * @return a Map between ascending sorted areas and the correspondent center
	 * of the respective rectangle computed.
	 */
	public static Map<CvPoint, Integer> computeCentroids(IplImage bwImg, int MAX_RECT_AREA) {
		
		Map<CvPoint,Integer> centroids = new HashMap<CvPoint,Integer>();
		
		CvMemStorage	storage	= CvMemStorage.create();
		CvSeq			contour	= new CvSeq(null);

		cvFindContours(bwImg, storage, contour, Loader.sizeof(CvContour.class), CV_RETR_LIST, CV_CHAIN_APPROX_SIMPLE);

		while (contour != null && !contour.isNull()) {
			
			CvPoint centroid = new CvPoint();

			if (contour.elem_size() > 0) {
				
				CvBox2D box = cvMinAreaRect2(contour, storage);
				
				if (box != null) {   
					
					CvPoint2D32f center = box.center();
					
					int area = (int) (box.size().width() * box.size().height());
					
					if( area < MAX_RECT_AREA && area > 0) {
						centroid.set(center,(byte)0);
						centroids.put(centroid,area);
					}
				}
				
			}
			
			contour = contour.h_next();
		}

		return centroids;
	}
	
}
